package  
{
	import net.flashpunk.Entity;
	import net.flashpunk.Graphic;
	import net.flashpunk.graphics.Stamp;
	import net.flashpunk.FP;
	import net.flashpunk.Sfx;
	import net.flashpunk.tweens.misc.VarTween;
	import net.flashpunk.utils.Ease;
	
	/**
	 * ...
	 * @author Switchbreak
	 */
	public class Boulder extends Entity 
	{
		
		[Embed(source = '../images/Boulder.png')] private static const BOULDER_IMAGE:Class;
		[Embed(source = '../snd/Boulder.mp3')] private static const BOULDER_SOUND:Class;
		
		public static const BOULDER_RADIUS:int = 10;
		
		private static const BOULDER_COLOR:uint = 0xCCCCCC;
		private static const GRAVITY:Number = 200;
		
		private var nStartX:int, nStartY:int;
		private var boulderSprite:Stamp;
		private var boulderSound:Sfx;
		private var quake:ScreenShake;
		
		private var velocityX:Number = 0;
		private var velocityY:Number = 0;
		
		public function Boulder( nSetStartX:int, nSetStartY:int )
		{
			nStartX = nSetStartX;
			nStartY = nSetStartY;
			
			//boulderSprite = Image.createCircle( BOULDER_RADIUS, BOULDER_COLOR );
			boulderSprite = new Stamp( BOULDER_IMAGE, -BOULDER_RADIUS, -BOULDER_RADIUS );
			boulderSound = new Sfx( BOULDER_SOUND );
			
			super(nStartX, nStartY, boulderSprite);
			
			quake = FP.world.add( new ScreenShake() ) as ScreenShake;
			
			layer = -1;
			type = "boulders";
			
			//boulderSprite.centerOO();
		}
		
		override public function update():void 
		{
			x += velocityX * FP.elapsed;
			y += velocityY * FP.elapsed;
			
			velocityY += GRAVITY * FP.elapsed;
			
			if ( y >= FP.height + BOULDER_RADIUS )
			{
				x = nStartX;
				y = nStartY;
				velocityX = velocityY = 0;
			}
			
			if ( velocityY > 0 )
			{
				var testPlatform:Platform = FP.world.nearestToEntity( "platforms", this, true ) as Platform;
				if ( testPlatform != null )
				{
					if ( distanceFrom( testPlatform, true ) <= BOULDER_RADIUS )
					{
						boulderSound.play( 0.5 );
						quake.Start();
						velocityY = -70;
						
						var targetX:int = x;
						if ( FP.random > 0.5 )
							targetX += 30;
						else
							targetX -= 30;
						
						var horizontalTween:VarTween = new VarTween( null, ONESHOT );
						horizontalTween.tween( this, "x", targetX, 0.7, Ease.sineOut );
						clearTweens();
						addTween( horizontalTween, true );
					}
				}
			}
			
			super.update();
		}
		
	}

}