package  
{
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Stamp;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.World;
	import net.flashpunk.FP;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	
	/**
	 * Game world object, initializes and manages all game objects
	 * 
	 * @author Switchbreak
	 */
	public class GameWorld extends World 
	{
		
        [Embed (source = "../xml/Level1.xml", mimeType = "application/octet-stream")] private static const LEVEL1:Class;
        [Embed (source = "../xml/Level2.xml", mimeType = "application/octet-stream")] private static const LEVEL2:Class;
        [Embed (source = "../xml/Level3.xml", mimeType = "application/octet-stream")] private static const LEVEL3:Class;
        [Embed (source = "../xml/Level4.xml", mimeType = "application/octet-stream")] private static const LEVEL4:Class;
        [Embed (source = "../xml/Level5.xml", mimeType = "application/octet-stream")] private static const LEVEL5:Class;
		
        [Embed (source = "../images/Mute.png")] private static const MUTE_IMAGE:Class;
        [Embed (source = "../images/Mute-Toggled.png")] private static const MUTE_TOGGLED:Class;
		
		private var nStageNumber:int;
		private var player:Player;
		private var startingPlatform:Platform = null;
		private var muteImage:Stamp;
		
		public var nSurvivorCount:int = 0;
		private var bMute:Boolean = false;
		
		public function GameWorld( nStage:int = 1 )
		{
			nStageNumber = nStage;
		}
		
		/**
		 * Initialize the game world
		 */
		override public function begin():void 
		{
			super.begin();
			
			//FP.console.enable();
			FP.screen.color = 0x288CFF;
			
			add( new Waterfall() );
			
			muteImage = new Stamp( MUTE_IMAGE, FP.width - 15, 5 );
			muteImage.scrollX = muteImage.scrollY;
			addGraphic( muteImage, 10 );
			
			// Load game data from XML file
			var levelXML:XML = null;
			switch( nStageNumber )
			{
				case 1:
					levelXML = new XML( new LEVEL1() );
					break;
				case 2:
					levelXML = new XML( new LEVEL2() );
					break;
				case 3:
					levelXML = new XML( new LEVEL3() );
					break;
				case 4:
					levelXML = new XML( new LEVEL4() );
					break;
				case 5:
					levelXML = new XML( new LEVEL5() );
					break;
			}
			
			if ( levelXML == null )
				throw( new Error( "Stage XML not found" ) );
			
			for each( var platformNode:XML in levelXML.platform )
			{
				var platform:Platform = add( new Platform( platformNode.@x, platformNode.@y, (platformNode.@goal == "true"), platformNode.@x2, platformNode.@y2, platformNode.@speed ) ) as Platform;
				
				if ( platformNode.@start == "true" )
					startingPlatform = platform;
				
				if ( platformNode.@survivor == "true" )
				{
					add( new Survivor( platform ) );
					nSurvivorCount++;
				}
			}
			
			for each( var boulderNode:XML in levelXML.boulder )
			{
				var boulder:Boulder = add( new Boulder( boulderNode.@x, boulderNode.@y ) ) as Boulder;
			}
			
			if ( startingPlatform == null )
				throw( new Error( "No starting platform specified" ) );
			
			player = add( new Player( startingPlatform, FadeOut, (nStageNumber > 1) ) ) as Player;
		}
		
		/**
		 * Handle game object updating
		 */
		override public function update():void 
		{
			// If the player hits the bottom of the screen, reset them
			if ( player.y >= FP.height || Input.pressed( Key.BACKSPACE ) )
			{
				remove( player );
				player = add( new Player( startingPlatform, FadeOut, (nStageNumber > 1) ) ) as Player;
			}
			
			if ( Input.pressed( Key.M ) || 
				(Input.mousePressed && Input.mouseX >= muteImage.x && Input.mouseX <= muteImage.x + 10
					&& Input.mouseY >= muteImage.y && Input.mouseY <= muteImage.y + 10) )
			{
				bMute = !bMute;
				
				if ( bMute )
				{
					muteImage.source = FP.getBitmap( MUTE_TOGGLED );
					FP.volume = 0;
				}
				else
				{
					muteImage.source = FP.getBitmap( MUTE_IMAGE );
					FP.volume = 1;
				}
			}
			
			super.update();
		}
		
		public function ShowWinScreen():void
		{
			var winScreen:WinScreen = new WinScreen();
			FP.world = winScreen;
		}
		
		public function FadeOut():void
		{
			add( new Fader( ShowWinScreen, 1, false, 0 ) );
		}
		
	}

}