package  
{
	import net.flashpunk.Entity;
	import net.flashpunk.Graphic;
	import net.flashpunk.graphics.Stamp;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.FP;
	import net.flashpunk.Sfx;
	import net.flashpunk.tweens.motion.LinearMotion;
	import net.flashpunk.utils.Ease;
	
	/**
	 * ...
	 * @author Switchbreak
	 */
	public class Platform extends Entity 
	{
		
		[Embed(source = '../images/Outcrop.png')] private static const PLATFORM_IMAGE:Class;
		[Embed(source = '../images/Helicopter.png')] private static const HELICOPTER_IMAGE:Class;
		[Embed(source = '../snd/Chopper.mp3')] private static const CHOPPER_SOUND:Class;
		
		private static const PLATFORM_WIDTH:int = 30;
		private static const PLATFORM_HEIGHT:int = 10;
		private static const PLATFORM_COLOR:uint = 0x888888;
		private static const GOAL_COLOR:uint = 0xCCCC00;
		
		private var platformSprite:Image;
		private var motionTween:LinearMotion;
		private var bFlip:Boolean = true;
		private var x1:int, x2:int, y1:int, y2:int;
		private var fSpeed:Number;
		private var tutorialText:String;
		
		public var survivor:Survivor = null;
		public var _bGoal:Boolean = false;
		
		public function Platform(nSetX:int, nSetY:int, bSetGoal:Boolean = false, nSetX2:int = 0, nSetY2:int = 0, fSetSpeed:Number = 10)
		{
			//platformSprite = Image.createRect( PLATFORM_WIDTH, PLATFORM_HEIGHT, 0xffffff );
			_bGoal = bSetGoal;
			
			if ( _bGoal )
			{
				platformSprite = new Image( HELICOPTER_IMAGE );
				
				x1 = nSetX;
				if( nSetX2 == 0 )
					x2 = nSetX;
				else
					x2 = nSetX2;
				
				y1 = nSetY;
				if( nSetY2 == 0 )
					y2 = nSetY - 10;
				else
					y2 = nSetY2;
				
				if ( x2 < x1 )
					platformSprite.flipped = true;
				
				if ( fSetSpeed == 0 )
					fSpeed = 10;
				else
					fSpeed = fSetSpeed;
				
				motionTween = new LinearMotion( FlipTween, LOOPING );
				motionTween.setMotionSpeed( x1, y1, x2, y2, fSpeed, Ease.sineInOut );
				addTween( motionTween, true );
			}
			else
			{
				platformSprite = new Image( PLATFORM_IMAGE );
				FP.world.add( new PlatformWater( nSetY - 1, nSetX, PLATFORM_WIDTH ) );
			}
			
			setHitbox( PLATFORM_WIDTH, PLATFORM_HEIGHT );
			
			super(nSetX, nSetY, platformSprite);
			
			active = _bGoal;
			type = "platforms";
		}
		
		override public function update():void 
		{
			if ( _bGoal )
			{
				x = motionTween.x;
				y = motionTween.y;
			}
			
			super.update();
		}
		
		public function FlyOff():void
		{
			motionTween.setMotion( x, y, -150, -150, 5.0, Ease.sineOut );
			var chopperSound:Sfx = new Sfx( CHOPPER_SOUND )
			chopperSound.play( 0.5 );
		}
		
		public function FlipTween():void
		{
			if ( bFlip )
			{
				motionTween.setMotionSpeed( x2, y2, x1, y1, fSpeed, Ease.sineInOut );
				if ( x1 < x2 )
					platformSprite.flipped = true;
				else
					platformSprite.flipped = false;
			}
			else
			{
				motionTween.setMotionSpeed( x1, y1, x2, y2, fSpeed, Ease.sineInOut );
				if ( x2 < x1 )
					platformSprite.flipped = true;
				else
					platformSprite.flipped = false;
			}
			
			bFlip = !bFlip;
		}
		
		public function get bGoal():Boolean
		{
			return _bGoal;
		}
		
	}

}