package  
{
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.Graphic;
	import net.flashpunk.graphics.Graphiclist;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.FP;
	import net.flashpunk.Sfx;
	import net.flashpunk.tweens.misc.ColorTween;
	import net.flashpunk.tweens.misc.VarTween;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	
	/**
	 * Player object, handles player input and movement
	 * 
	 * @author Switchbreak
	 */
	public class Player extends Entity 
	{
		
		[Embed(source = '../snd/Jump.mp3')] private static const JUMP_SOUND:Class;
		[Embed(source = '../snd/Rope.mp3')] private static const ROPE_SOUND:Class;
		[Embed(source = '../snd/Hurt.mp3')] private static const HURT_SOUND:Class;
		
		// Private constants
		private static const PLAYER_WIDTH:int = 5;
		private static const PLAYER_HEIGHT:int = 10;
		private static const PLAYER_COLOR:uint = 0xffffff;
		private static const ROPE_COLOR:uint = 0xffffff;
		private static const ROPE_LENGTH:int = 20;
		private static const ROTATE_SPEED:Number = 270;
		private static const SWING_BOUND:Number = 90;
		private static const SWING_OFFSET:Number = 20;
		private static const RELEASE_SPEED:Number = 100;
		private static const GRAPPLE_SPEED:Number = 40;
		private static const GRAVITY:Number = 200;
		private static const X_BOOST:Number = 0;
		private static const Y_BOOST:Number = -30;
		
		// Player graphics
		private var playerSprite:Image;
		private var rope:Image;
		private var tutText:Text, tutText1:Text, tutText2:Text, tutText3:Text;
		private var jumpSound:Sfx, ropeSound:Sfx, hurtSound:Sfx;
		
		// Collected survivors
		private var survivors:Vector.<Survivor>;
		
		// Swinging state variables
		private var fSwingAngle:Number = 0;
		private var nSwingDirection:int = -1;
		private var fRadius:Number = ROPE_LENGTH;
		private var _bSwinging:Boolean = true;
		private var bInbound:Boolean = true;
		private var bLocked:Boolean = false;
		private var winCallback:Function;
		private var bTutorial:Boolean = false;
		private var bTutorial1:Boolean = false;
		private var bTutorial2:Boolean = false;
		private var bTutorial3:Boolean = false;
		private var grabPlatform:Platform = null;
		
		// Release state variables
		private var velocity:Point = new Point();
		
		/**
		 * Constructor, initializes player data
		 * 
		 * @param	platform	Platform which the player starts out hanging from
		 */
		public function Player( platform:Platform, setWinCallback:Function = null, bSetTutorial:Boolean = false ) 
		{
			// Initialize survivor list
			survivors = new Vector.<Survivor>();
			
			winCallback = setWinCallback;
			bTutorial = bSetTutorial;
			bTutorial1 = bSetTutorial;
			bTutorial2 = bSetTutorial;
			bTutorial3 = bSetTutorial;
			
			// Initialize graphic
			playerSprite = Image.createRect( PLAYER_WIDTH, PLAYER_HEIGHT, PLAYER_COLOR );
			playerSprite.smooth = false;
			playerSprite.x = 0;
			playerSprite.y = ROPE_LENGTH;
			playerSprite.centerOO();
			
			rope = Image.createRect( 1, 1 );
			rope.scaleY = fRadius;
			
			jumpSound = new Sfx( JUMP_SOUND );
			ropeSound = new Sfx( ROPE_SOUND );
			hurtSound = new Sfx( HURT_SOUND );
			
			tutText = new Text( "Press Space to release", FP.halfWidth, FP.height - 20 );
			tutText.relative = false;
			tutText.centerOO();
			tutText.visible = !bTutorial;
			
			tutText1 = new Text( "Release the rope and grab in \nthe air to jump to this platform", 190, 120 );
			tutText1.relative = false;
			tutText1.size = 8;
			tutText1.centerOO();
			tutText1.visible = !bTutorial1;
			
			tutText2 = new Text( "Good! Now release at the end of your \nswing to jump up and rescue this climber", 220, 70 );
			tutText2.relative = false;
			tutText2.size = 8;
			tutText2.centerOO();
			tutText2.alpha = 0;
			tutText2.visible = !bTutorial2;
			
			tutText3 = new Text( "Gather all the climbers and \nbring them to the chopper", 90, 40 );
			tutText3.relative = false;
			tutText3.size = 8;
			tutText3.centerOO();
			tutText3.alpha = 0;
			tutText3.visible = !bTutorial3;
			
			grabPlatform = platform;
			
			// Set starting position to hang from initially selected platform
			x = grabPlatform.x + grabPlatform.halfWidth;
			y = grabPlatform.y + grabPlatform.height;
			
			super( x, y, new Graphiclist( rope, playerSprite, tutText, tutText1, tutText2, tutText3 ) );
		}
		
		/**
		 * Updates player state and handles input every frame
		 */
		override public function update():void 
		{
			//
			// Handle player physics
			//
			
			if ( _bSwinging )
			{
				if ( !bLocked )
				{
					var fSwingAmount:Number = nSwingDirection * ROTATE_SPEED * FP.elapsed;
					
					if ( bInbound )
					{
						fSwingAmount *= ((SWING_BOUND - fSwingAngle + 10) / SWING_BOUND);
						fSwingAmount *= (( -SWING_BOUND - fSwingAngle - 10) / -SWING_BOUND);
					}
					
					fSwingAngle += fSwingAmount;
					fSwingAngle %= 360;
					playerSprite.angle = fSwingAngle;
				}
				
				// Swinging physics
				x = grabPlatform.x + grabPlatform.halfWidth;
				y = grabPlatform.y + grabPlatform.height;
				
				if ( fSwingAngle * nSwingDirection > SWING_BOUND )
				{
					if ( bInbound )
					{
						fSwingAngle = SWING_BOUND * nSwingDirection * 2 - fSwingAngle;
						nSwingDirection = -nSwingDirection;
					}
				}
				else if( fSwingAngle * nSwingDirection > 0 )
				{
					bInbound = true;
				}
				
				if ( fRadius < ROPE_LENGTH )
				{
					fRadius += GRAPPLE_SPEED * FP.elapsed;
					if ( fRadius > ROPE_LENGTH )
						fRadius = ROPE_LENGTH;
					rope.scaleY = fRadius;
				}
				else if ( fRadius > ROPE_LENGTH )
				{
					fRadius -= GRAPPLE_SPEED * FP.elapsed;
					if ( fRadius < ROPE_LENGTH )
						fRadius = ROPE_LENGTH;
					rope.scaleY = fRadius;
				}
				
				rope.angle = fSwingAngle;
				FP.angleXY( playerSprite, fSwingAngle + 270, fRadius, -playerSprite.originX, -playerSprite.originY );
			}
			else
			{
				if ( !bLocked )
				{
					fSwingAngle += nSwingDirection * ROTATE_SPEED * FP.elapsed;
					fSwingAngle %= 360;
					playerSprite.angle = fSwingAngle;
				}
				
				// Flying physics
				x += velocity.x * FP.elapsed;
				y += velocity.y * FP.elapsed;
				
				velocity.y += GRAVITY * FP.elapsed;
				
				var testBoulder:Boulder = FP.world.nearestToEntity( "boulders", this ) as Boulder;
				if ( testBoulder != null )
				{
					if ( distanceFrom( testBoulder ) <= Boulder.BOULDER_RADIUS )
					{
						if( !bLocked )
							hurtSound.play();
						bLocked = true;
						_bSwinging = false;
						y = testBoulder.y + Boulder.BOULDER_RADIUS;
						velocity.x = velocity.y = 0;
					}
				}
			}
			
			var hangX:int = x + playerSprite.x + playerSprite.originX;
			var hangY:int = y + playerSprite.y + playerSprite.originY;
			
			for each( var survivor:Survivor in survivors )
			{
				if ( !bLocked )
				{
					survivor.x = hangX;
					survivor.y = hangY;
					
					hangX = survivor.x + survivor.survivorSprite.x + survivor.survivorSprite.originX;
					hangY = survivor.y + survivor.survivorSprite.y + survivor.survivorSprite.originY;
				}
				else
				{
					if ( y > (survivor.y + survivor.survivorSprite.y + survivor.survivorSprite.originY) )
					{
						survivor.y += y - (survivor.y + survivor.survivorSprite.y + survivor.survivorSprite.originY);
					}
				}
			}
			
			//
			// Handle player input
			//
			
			// Space bar to release or grab on
			if ( !bLocked && Input.pressed( Key.SPACE ) )
			{
				if ( !bTutorial )
				{
					bTutorial = true;
					
					var tutorialFader:VarTween = new VarTween( null, ONESHOT );
					tutorialFader.tween( tutText, "alpha", 0, 5.0 );
					addTween( tutorialFader, true );
				}
				
				// If currently swinging, then release, if flying try to grab
				if ( _bSwinging )
				{
					jumpSound.play( 0.5 );
					
					_bSwinging = false;
					rope.visible = false;
					tutText.text = "Press Space to grab";
					
					// Get the release velocity vector based on the current swing angle
					FP.angleXY( velocity, fSwingAngle, RELEASE_SPEED, X_BOOST, Y_BOOST * nSwingDirection );
					
					velocity.x *= nSwingDirection;
					velocity.y *= nSwingDirection;
					
					x += playerSprite.x;
					y += playerSprite.y;
					
					playerSprite.x = playerSprite.y = 0;
					
					if ( survivors.length > 0 )
					{
						
						for each( survivor in survivors )
						{
							survivor.LerpAngle( 0, 0.2 );
						}
					}
				}
				else
				{
					grabPlatform = FP.world.nearestToEntity( "platforms", this, true ) as Platform;
					
					if ( grabPlatform != null )
					{
						if ( distanceFrom( grabPlatform, true ) <= ROPE_LENGTH )
						{
							var targetX:int = grabPlatform.x + grabPlatform.halfWidth;
							var targetY:int = grabPlatform.y + grabPlatform.height;
							fRadius = distanceToPoint( targetX, targetY );
							
							ropeSound.play( 0.5 );
							
							if ( !bTutorial1 )
							{
								bTutorial1 = true;
								
								var tutorialFader1:VarTween = new VarTween( null, ONESHOT );
								tutorialFader1.tween( tutText1, "alpha", 0, 1.0 );
								addTween( tutorialFader1, true );
								
								var tutorialFader2:VarTween = new VarTween( null, ONESHOT );
								tutorialFader2.tween( tutText2, "alpha", 1, 1.0 );
								addTween( tutorialFader2, true );
							}
							
							_bSwinging = true;
							bInbound = false;
							rope.visible = true;
							tutText.text = "Press Space to release"
							
							fSwingAngle = FP.angle( targetX, targetY, x, y ) - 270;
							
							if ( x < targetX )
								nSwingDirection = 1;
							else
								nSwingDirection = -1;
							
							playerSprite.angle = fSwingAngle;
							rope.angle = fSwingAngle;
							rope.scaleY = fRadius;
							FP.angleXY( playerSprite, fSwingAngle + 270, fRadius, -playerSprite.originX, -playerSprite.originY );
							
							x = targetX;
							y = targetY;
							
							if ( grabPlatform.survivor != null )
							{
								if ( !bTutorial2 )
								{
									bTutorial2 = true;
									
									tutorialFader2 = new VarTween( null, ONESHOT );
									tutorialFader2.tween( tutText2, "alpha", 0, 1.0 );
									addTween( tutorialFader2, true );
									
									var tutorialFader3:VarTween = new VarTween( null, ONESHOT );
									tutorialFader3.tween( tutText3, "alpha", 1, 1.0 );
									addTween( tutorialFader3, true );
								}
								
								grabPlatform.survivor.player = this;
								grabPlatform.survivor.fSwingAngle = fSwingAngle;
								survivors.push( grabPlatform.survivor );
								grabPlatform.survivor = null;
							}
							
							var hangAngle:Number = fSwingAngle;
							for( var i:int = 0; i < survivors.length; i++ )
							{
								hangAngle -= SWING_OFFSET * nSwingDirection;
								//survivors[i].fSwingAngle = hangAngle;
								survivors[i].LerpAngle( hangAngle, 0.1 );
								survivors[i].nSwingDirection = nSwingDirection;
								survivors[i].bInbound = false;
								survivors[i].layer = i;
							}
							
							if ( grabPlatform.bGoal && survivors.length >= (world as GameWorld).nSurvivorCount )
							{
								if ( !bTutorial3 )
								{
									bTutorial3 = true;
									
									tutorialFader3 = new VarTween( null, ONESHOT );
									tutorialFader3.tween( tutText3, "alpha", 0, 1.0 );
									addTween( tutorialFader3, true );
								}
								
								grabPlatform.FlyOff();
								
								if ( winCallback != null )
									winCallback();
							}
						}
						else
							grabPlatform = null;
					}
				}
			}
			
			super.update();
		}
		
		/**
		 * Put all gathered survivors back on their platforms when the player is removed from the world
		 */
		override public function removed():void 
		{
			for each( var survivor:Survivor in survivors )
			{
				survivor.platform = survivor.platform;
				survivor.player = null;
			}
			
			super.removed();
		}
		
		//
		// Accessors
		//
		
		/**
		 * Get the last platform from which the survivor has been hanging
		 */
		public function get bSwinging():Boolean
		{
			return _bSwinging;
		}
		
	}

}