package  
{
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.Graphic;
	import net.flashpunk.graphics.Graphiclist;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.FP;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	
	/**
	 * Survivor entity, rescuable by players
	 * 
	 * @author Switchbreak
	 */
	public class Survivor extends Entity 
	{
		
		// Private constants
		private static const SURVIVOR_WIDTH:int = 5;
		private static const SURVIVOR_HEIGHT:int = 10;
		private static const SURVIVOR_COLOR:uint = 0x99AA00;
		private static const ROTATE_SPEED:Number = 270;
		private static const SWING_BOUND:Number = 90;
		private static const ROPE_COLOR:uint = 0xffffff;
		
		public static const ROPE_LENGTH:int = 15;
		
		// Player graphics
		private var _survivorSprite:Image;
		private var rope:Image;
		private var _platform:Platform;
		
		public var player:Player = null;
		public var nSwingDirection:int = 1;
		public var bInbound:Boolean = false;
		
		// Swinging state variables
		private var _fSwingAngle:Number = 0;
		private var fLerpTimer:Number = 0;
		private var fLerpAmount:Number;
		
		/**
		 * Constructor, initializes survivor data
		 * 
		 * @param	setPlatform		Platform which the survivor starts out hanging from
		 */
		public function Survivor( setPlatform:Platform ) 
		{
			// Initialize graphic
			_survivorSprite = Image.createRect( SURVIVOR_WIDTH, SURVIVOR_HEIGHT, SURVIVOR_COLOR );
			with( _survivorSprite )
			{
				smooth = false;
				x = 0;
				y = ROPE_LENGTH;
				centerOO();
			}
			
			rope = Image.createRect( 1, ROPE_LENGTH );
			
			platform = setPlatform;
			
			super( x, y, new Graphiclist( rope, _survivorSprite ) );
		}
		
		override public function update():void 
		{
			if ( fLerpTimer > 0 )
			{
				fSwingAngle += fLerpAmount * FP.elapsed;
				fLerpTimer -= FP.elapsed;
			}
			
			if ( player != null )
			{
				if ( player.bSwinging )
				{
					var fSwingAmount:Number = nSwingDirection * ROTATE_SPEED * FP.elapsed;
					
					if ( bInbound )
					{
						fSwingAmount *= ((SWING_BOUND - fSwingAngle + 10) / SWING_BOUND);
						fSwingAmount *= (( -SWING_BOUND - fSwingAngle - 10) / -SWING_BOUND);
					}
					
					fSwingAngle += fSwingAmount;
					fSwingAngle %= 360;
					
					if ( fSwingAngle * nSwingDirection > SWING_BOUND )
					{
						if ( bInbound )
						{
							fSwingAngle = SWING_BOUND * nSwingDirection * 2 - fSwingAngle;
							nSwingDirection = -nSwingDirection;
						}
					}
					else if( fSwingAngle * nSwingDirection > 0 )
					{
						bInbound = true;
					}
				}
			}
			
			super.update();
		}
		
		public function LerpAngle( fSetAngle:Number, fDuration:Number ):void
		{
			fLerpAmount = (fSetAngle - fSwingAngle) / fDuration;
			fLerpTimer = fDuration;
		}
		
		//
		// Accessors
		//
		
		/**
		 * Get the last platform from which the survivor has been hanging
		 */
		public function get platform():Platform
		{
			return _platform;
		}
		
		/**
		 * Hangs the survivor from a specified platform
		 */
		public function set platform( setPlatform:Platform ):void
		{
			// Set the selected platform
			_platform = setPlatform;
			_platform.survivor = this;
			
			// Set starting position to hang from selected platform
			x = _platform.x + _platform.halfWidth;
			y = _platform.y + _platform.height;
			fSwingAngle = 0;
			fLerpTimer = 0;
			bInbound = false;
		}
		
		public function get fSwingAngle():Number
		{
			return _fSwingAngle;
		}
		
		public function set fSwingAngle( fSetAngle:Number ):void
		{
			_fSwingAngle = fSetAngle;
			
			_survivorSprite.angle = _fSwingAngle;
			rope.angle = fSwingAngle;
			FP.angleXY( _survivorSprite, _fSwingAngle + 270, ROPE_LENGTH, -_survivorSprite.originX, -_survivorSprite.originY );
		}
		
		public function get survivorSprite():Image
		{
			return _survivorSprite;
		}
		
	}

}